--[[
Broker_PlayTimer
Displays how much time you've spent playing World of Warcraft today.

Copyright (c) 2009-2011, Scott Forbes
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]



--------------------------------------------------------------------------------
-- Start by creating our add-on, data object, locale and minimap icon.
--------------------------------------------------------------------------------
assert(LibStub, "Broker_PlayTimer requires LibStub")
local addon = LibStub("AceAddon-3.0"):NewAddon("Broker_PlayTimer", 'AceTimer-3.0', 'LibSink-2.0')
local dataobj   = LibStub("LibDataBroker-1.1"):NewDataObject(addon.name, {
	type 	= "data source",
	icon 	= [[Interface\Icons\Spell_Holy_BorrowedTime]],
	label 	= addon.name,
	text 	= "0m",
})
local L 		= LibStub("AceLocale-3.0"):GetLocale(addon.name)
local icon		= LibStub("LibDBIcon-1.0")




--------------------------------------------------------------------------------
-- Local variables.
--------------------------------------------------------------------------------

local db, sessionTimer, selectedTimer, startOfNewDay
local player, realm = UnitName("player"), GetRealmName()
local showingTooltip, requestingTimePlayed = false, false
local playedTotal, playedLevel = 0, 0
local ORIG_ChatFrame_DisplayTimePlayed = ChatFrame_DisplayTimePlayed

local RED		= {r=1.0, g=0.0, b=0.0}
local GREEN		= {r=0.0, g=1.0, b=0.0}
local BLUE		= {r=0.0, g=0.0, b=1.0}
local CYAN		= {r=0.0, g=1.0, b=1.0}
local MAGENTA	= {r=1.0, g=0.0, b=1.0}
local YELLOW	= {r=1.0, g=1.0, b=0.0}
local BLACK		= {r=0.0, g=0.0, b=0.0}
local WHITE 	= {r=1.0, g=1.0, b=1.0}
local PURPLE	= {r=1.0, g=0.5, b=1.0}
local ORANGE	= {r=1.0, g=0.5, b=.25}
local LTBLUE	= {r=0.5, g=0.5, b=1.0}

local ALWAYS			= 1
local IF_BOTH_PLAYED	= 2
local NEVER				= 3

local timerDisplayed
local SESSION_TIMER		= 1
local TODAY_TIMER		= 2
local PLAYER_TIMER		= 3

local SINGULAR			= true
local PLURAL			= false

local SHORT_TIME_FORMAT	= 1
local MED_TIME_FORMAT	= 2
local LONG_TIME_FORMAT	= 3

local timerNames = {
		L["Session"],
		L["Today"],
		player,
	}

local WHISPER			= 1
local RAID_WARNING		= 2
local CHAT_FRAME		= 3
local DIALOG_BOX		= 4
local LIBSINK			= 5

local factionColor = { 
	["Alliance"] = CYAN, 
	["Horde"]    = RED 
	}




--------------------------------------------------------------------------------
-- Local functions.
--------------------------------------------------------------------------------


-- thisRealm() and thisPlayer() are convenient shorthand for the database tables
-- matching the current realm and player.

local function thisRealm()
	return db.global.realmTable[realm]
end

local function thisPlayer()
	if thisRealm() then
		return thisRealm().playerTable[player]
	end
end



-- tomorrow() and yesterday() are used to determine when timers need to be reset.
-- These functions return a table that can be compared directly to time().

local function tomorrow()
	local d = date("*t")
	
	if d.hour >= db.profile.firstHourOfNewDay then
		d.day = d.day + 1
	end
	
	d.hour, d.min, d.sec = db.profile.firstHourOfNewDay, 0, 0	
	return time(d)
end

local function yesterday()
	local d = date("*t")
	
	if d.hour < db.profile.firstHourOfNewDay then
		d.day = d.day - 1
	end
	
	d.hour, d.min, d.sec = db.profile.firstHourOfNewDay, 0, 0	
	return time(d)
end



-- ResetDayTimers() resets "time played today" to zero, and deletes all timers
-- that were tallying how much an individual character had been played today.

local function ResetDayTimers()
	db.global.todayTimer = 0
	db.global.realmTable = {}
end



-- CreateMyTimers() creates timers for the current realm and player, if they
-- don't already exist.

local function CreateMyTimers()
	if not thisRealm() then
		db.global.realmTable[realm] = { 
			realmTimer = 0, 
			playerTable = {} 
		}
	end
	
	if not thisPlayer() then
		thisRealm().playerTable[player] = { 
			playerTimer = 0, 
			playerFaction = UnitFactionGroup("player")
		}
	end
end



-- If the showColorText preference is set to true, colorizeText() does what it 
-- says on the tin.  Otherwise, it simply returns the passed-in text string.

local function colorizeText(theText, theColor)
	if db.profile.showColorText and theColor then
		local r, g, b = theColor.r*255, theColor.g*255, theColor.b*255
		return string.format("|cff%02x%02x%02x%s|r", r, g, b, theText)
	else
		return theText
	end
end


-- formatTimeWithColor() formats the time in the specified format and color --
-- "1 hour, 30 minutes", "1hr, 30min", or "1h 30m", with the numerals in color.
--   "theTime" is the number of minutes (e.g., "60" = one hour).
--   "theFormat" is SHORT_TIME_FORMAT, MED_TIME_FORMAT, or LONG_TIME_FORMAT.
--   "theColor" is a table with r, g and b entries, or nil for no color.

local function formatTimeWithColor(theTime, theFormat, theColor)
	theTime = theTime or 0
	theFormat = theFormat or db.profile.timeFormat
	local days, hours, minutes, dayString, hourString, minString, formatString
	local result = ""
	
	days = floor(theTime/1440)
	hours = floor((theTime % 1440)/60)
	minutes = theTime % 60
	
	if theFormat == SHORT_TIME_FORMAT then
		-- Short time format is a special case, because it displays "1h 0m"
		-- instead of just "1h".
		dayString, hourString, minString = L["d"], L["h"], L["m"]

		if days > 0 then
			formatString = colorizeText("%d", theColor)..dayString.." "
			result = (formatString):format(days)
		end
		
		if days > 0 or hours > 0 then
			formatString = colorizeText("%d", theColor)..hourString.." "
			result = result..(formatString):format(hours)
		end
		
		formatString = colorizeText("%d", theColor)..minString
		result = result..(formatString):format(minutes)
	else
		if theFormat == MED_TIME_FORMAT then
			dayString	= { [SINGULAR]=L["day"],	[PLURAL]=L["days"] }
			hourString	= { [SINGULAR]=L["hr"],		[PLURAL]=L["hrs"] }
			minString	= { [SINGULAR]=L["min"],	[PLURAL]=L["mins"] }
		else		-- LONG_TIME_FORMAT
			dayString	= { [SINGULAR]=L["day"],	[PLURAL]=L["days"] }
			hourString	= { [SINGULAR]=L["hour"],	[PLURAL]=L["hours"] }
			minString	= { [SINGULAR]=L["minute"],	[PLURAL]=L["minutes"] }		
		end
		
		if days > 0 then
			formatString = colorizeText("%d", theColor).." "..dayString[days==1]
			result = (formatString):format(days)
			if hours > 0 or minutes > 0 then result = result..", " end
		end
		
		if hours > 0 then
			formatString = colorizeText("%d", theColor).." "..hourString[hours==1]
			result = result..(formatString):format(hours)
			if minutes > 0 then result = result..", " end
		end
		
		if minutes > 0 or theTime == 0 then
			formatString = colorizeText("%d", theColor).." "..minString[minutes==1]
			result = result..(formatString):format(minutes)
		end		
	end	
	
	return result
end



-- formatTime() formats the time with a default color of WHITE.

local function formatTime(theTime, theFormat)
	return formatTimeWithColor(theTime, theFormat, WHITE)
end



-- selectedTimer() returns the current value of the specified timer.
--   "theTimer" is SESSION_TIMER, TODAY_TIMER, or PLAYER_TIMER.

local function selectedTimer(theTimer)
	local timerOptions = {
		sessionTimer, 
		db.global.todayTimer, 
		thisPlayer().playerTimer
	}
	return timerOptions[theTimer]
end
	
	

-- UpdateStatusBarText() is called when the time changes, or when the player
-- changes the displayed timer, changes the preferred time format, or toggles
-- the preferences for using colors or labels.

local function UpdateStatusBarText()
	local timerName = ""
	if db.profile.showTimerName then
		timerName = timerNames[timerDisplayed]..": "
	end
	dataobj.text = timerName..formatTime(selectedTimer(timerDisplayed))
end



-- ChatFrame_DisplayTimePlayed() overrides the default Blizzard method for
-- displaying the results of the "/played" command, and -- if PlayTimer was the
-- source of the command -- caches the results and suppresses the chat display.
-- This technique is borrowed from FuBar_PlayFu's solution to the same problem.

function ChatFrame_DisplayTimePlayed(theFrame, totalTime, levelTime)
	if requestingTimePlayed then
		requestingTimePlayed = false
		playedTotal, playedLevel = totalTime/60, levelTime/60
	else
		ORIG_ChatFrame_DisplayTimePlayed(theFrame, totalTime, levelTime)
	end
end



-- hideMyWhisper() removes one "outbound" whisper from the user's chat frame,
-- to prevent the user from seeing two copies of whispered reminders.

local function hideMyWhisper(self, event, message)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", hideMyWhisper)
	return true
end




--------------------------------------------------------------------------------
-- PlayTimer and DataObj methods.
--------------------------------------------------------------------------------


-- addon:OnEnable() sets up our options and variables, and then initializes
-- the timers and data displays.

function addon:OnEnable()

	-- Set up saved variables with reasonable defaults.
	db = LibStub("AceDB-3.0"):New(addon.name.."DB", {
		global = {
			todayTimer 			= 0,
			lastTimerUpdate 	= time(),
			realmTable 			= {},
			
			remindersEnabled	= false,
			reminderTimer		= TODAY_TIMER,
			remindMinutes		= 60,
			snoozeEnabled		= false,
			snoozeMinutes		= 30,
			reminderType		= RAID_WARNING,
			reminderText		= L["Reminder: You have been playing for %s."],
		},
		profile = {
			minimap 			= { hide = true },
			timeFormat			= SHORT_TIME_FORMAT,
			showColorText 		= true,
			showTimerName		= true,
			showFactions 		= IF_BOTH_PLAYED,
			defaultTimer 		= TODAY_TIMER,
			firstHourOfNewDay	= 4,
			sinkOptions			= { sink20OutputSink = "UIErrorsFrame" },
		},
	})
	
	-- Connect LibDBIcon and LibSink to their saved variables.
	icon:Register(addon.name, dataobj, db.profile.minimap)
	self:SetSinkStorage(db.profile.sinkOptions)

	-- Use the "## Notes:" from our TOC file to describe the add-on.
	local locale, noteString = GetLocale()
	if locale == "enUS" then
		noteString = "Notes"
	else
		noteString = "Notes-"..locale
	end
	local notes = GetAddOnMetadata(addon.name, noteString) or GetAddOnMetadata(addon.name, "Notes")


	-- As far as I know Broker_Currency, by Toadkiller of Proudmoore, is the
	-- only other addon that creates a "Broker" menu within the Blizzard
	-- options dialog and then installs its own options as a sub-menu... but
	-- I like the idea and hope it catches on, so I'm doing the same.
	
	local brokerOptions = LibStub("AceConfigRegistry-3.0"):GetOptionsTable("Broker", "dialog", "LibDataBroker-1.1")
	if (not brokerOptions) then
		brokerOptions = {
			type = "group",
			name = "Broker",
			desc = "Configuration options for Broker plugins.  Please select a plugin from the submenu.",
			args = {
			}
		}
		LibStub("AceConfig-3.0"):RegisterOptionsTable("Broker", brokerOptions)
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Broker", "Broker")
	end
	
	
	-- Options.
	local optionsTable = {
		name = L[addon.name],
		handler = addon,
		type = 'group',
		args = {
			addonDescription = {
				type = 'description',
				name = notes,
				order = 0,
			},			
			displayOptions = {
				name = L["Display"],
				type = 'header',
				order = 10,
			},
			minimapToggle = {
				type = 'toggle',
				name = L["Show Minimap Icon"],
				desc = "",
				width = 'full',
				get = function(_) return (not db.profile.minimap.hide) end,
				set = function(_, value) 
					db.profile.minimap.hide = not value
					if db.profile.minimap.hide then
						icon:Hide(addon.name)
					else
						icon:Show(addon.name)
					end
				end,
				order = 15,
			},
			timeDisplay = {
				name = '',
				type = 'group',
				guiInline = true,
				order = 20,
				args = {
					timeFormat = {
						type = 'select',
						style = 'dropdown',
						values = {						-- "83" == 1 hour, 23 minutes
							[SHORT_TIME_FORMAT]	= formatTime(83, SHORT_TIME_FORMAT),
							[MED_TIME_FORMAT]	= formatTime(83, MED_TIME_FORMAT),
							[LONG_TIME_FORMAT]	= formatTime(83, LONG_TIME_FORMAT),
						},
						name = L["Time format:"],
						desc = L["Times will be displayed in the selected format."],
						get = function(_) return db.profile.timeFormat end,
						set = function(_, value) 
							db.profile.timeFormat = value
							UpdateStatusBarText()
						end,
						order = 20,
					},
					colorText = {
						type = 'toggle',
						name = L["Use color"],
						desc = L["Controls the use of color in the tooltip and on the status bar display."],
						get = function(_) return db.profile.showColorText end,
						set = function(_, value) 
							db.profile.showColorText = value
							UpdateStatusBarText()
						end,
						order = 30,
					},
				},
			},
			defaultStatusbar = {
				name = '',
				type = 'group',
				guiInline = true,
				order = 40,
				args = {
					defaultTimer = {
						type = 'select',
						style = 'dropdown',
						name = L["Default status bar timer:"],
						desc = L["The selected timer will be displayed in the status bar when the player initially enters the realm."],
						values = { 
							L["Time played this session"], 
							L["Total played today"], 
							L["This character today"]
						},
						get = function(_) return db.profile.defaultTimer end,
						set = function(_, value) 
							db.profile.defaultTimer = value
							timerDisplayed = value
							UpdateStatusBarText()
						end,
						order = 40,
					},
					timerName = {
						type = 'toggle',
						name = L["Show name of timer"],
						desc = L["Show the timer's name in the status bar display."],
						get = function(_) return db.profile.showTimerName end,
						set = function(_, value) 
							db.profile.showTimerName = value
							UpdateStatusBarText()					
							end,
						order = 50,
					},
				},
			},
			timerReset = {
				name = '',
				type = 'group',
				guiInline = true,
				order = 60,
				args = {
					newDayStart = {
						type = 'select',
						style = 'dropdown',
						name = L["Reset timers at:"],
						desc = L["Player, realm and faction timers will be reset to zero minutes at the selected time of day."],
						values = { 
							L["midnight"], 
							L["1:00 AM"], 
							L["2:00 AM"], 
							L["3:00 AM"], 
							L["4:00 AM"], 
							L["5:00 AM"], 
							L["6:00 AM"] 
						},
						get = function(_) return db.profile.firstHourOfNewDay + 1 end,
						set = function(_, value) db.profile.firstHourOfNewDay = value - 1 end,
						order = 60,
					},
					resetNow = {
						type = 'execute',
						name = L["Reset Now"],
						desc = L["Reset player, realm and faction timers to zero minutes."],
						func = function(_)
							ResetDayTimers()
							CreateMyTimers()
							UpdateStatusBarText()
						end,
						order = 70,
					},
				},
			},
			showFactions = {
				type = 'select',
				style = 'dropdown',
				name = L["Show Alliance / Horde timers:"],
				desc = L["Controls whether to show Alliance and Horde timers in the tooltip."],
				values = { 
					L["Always"],
					L["Only if I've played both"], 
					L["Never"] },
				get = function(_) return db.profile.showFactions end,
				set = function(_, value) db.profile.showFactions = value end,
				order = 80,
			},
			reminderOptions = {
				type = "header",
				name = L["Reminders"],
				order = 200,
			},
			remindToggle = {
				type = 'toggle',
				name = L["Enable Reminders"],
				desc = "",
				width = "full",
				get = function(_) return db.global.remindersEnabled end,
				set = function(_, value) db.global.remindersEnabled = value end,
				order = 210,
			},
			reminderTimer = {
				name = '',
				type = 'group',
				guiInline = true,
				order = 220,
				args = {
					remindTimer = {
						type = 'select',
						style = 'dropdown',
						name = L["Remind me when:"],
						values = { 
							L["Time played this session"], 
							L["Total played today"], 
							L["This character today"] 
						},
						desc = "",
						get = function(_) return db.global.reminderTimer end,
						set = function(_, value) db.global.reminderTimer = value end,
						order = 220,
						disabled = function(_) return not db.global.remindersEnabled end,
					},
					remindAfter = {
						type = 'range',
						name = L["exceeds (minutes):"],
						desc = L["Remind me after I've played for this length of time in minutes."],
						min = 15,
						max = 240,
						step = 15,
						get = function(_) return db.global.remindMinutes end,
						set = function(_, value) db.global.remindMinutes = value end,
						order = 230,
						disabled = function(_) return not db.global.remindersEnabled end,
					},
				},
			},
			snoozeTimer = {
				name = '',
				type = 'group',
				guiInline = true,
				order = 240,
				args = {
					snoozeToggle = {
						type = 'toggle',
						name = L["Remind me again"],
						desc = "",
						get = function(_) return db.global.snoozeEnabled end,
						set = function(_, value) db.global.snoozeEnabled = value end,
						order = 240,
						disabled = function(_) return not db.global.remindersEnabled end,
					},
					snoozeAfter = {
						type = 'range',
						name = L["every (minutes):"],
						desc = L["Snooze and remind me again after I've played for this length of time in minutes."],
						min = 15,
						max = 240,
						step = 15,
						get = function(_) return db.global.snoozeMinutes end,
						set = function(_, value) db.global.snoozeMinutes = value end,
						order = 250,
						disabled = function(_) 
							return not (db.global.snoozeEnabled and db.global.remindersEnabled)
						end,
					},
				},
			},
			reminderType = {
				type = 'select',
				style = 'radio',
				name = L["Send my reminders using:"],
				desc = L["The reminder will be delivered via the specified channel."],
				values = { 
					L["Whisper"], 
					L["Raid Warning"],
					L["Chat Message"],
					L["Dialog Box"],
					L["Custom"]
				},
				get = function(_) return db.global.reminderType end,
				set = function(_, value) db.global.reminderType = value end,
				order = 260,
				disabled = function(_) return not db.global.remindersEnabled end,
			},
			reminderExample = {
				type = 'execute',
				name = L["Example"],
				desc = L["Show an example of how the reminder will appear."],
				func = function(_)
					addon:ShowReminder()
				end,
				order = 270,
			},
			reminderText = {
				type = 'input',
				name = L["Reminder text:"],
				width = 'full',
				multiline = true,
				get = function(_) return db.global.reminderText end,
				set = function(_, value) db.global.reminderText = value end,
				order = 280,
				disabled = function(_) return not db.global.remindersEnabled end,
			},
			reminderDescription = {
				type = 'description',
				name = L["Use the string '%s' (without the quotes) to display the timer duration."],
				order = 290,
			},
		},
	}
	
	-- Set up the LibSink options table.
	optionsTable.args.output = addon:GetSinkAce3OptionsDataTable()
	optionsTable.args.output.inline = true
	optionsTable.args.output.order = 270
	optionsTable.args.output.hidden = function(_) return not(db.global.reminderType == LIBSINK) end
	optionsTable.args.output.disabled = function(_) return not db.global.remindersEnabled end
	
	-- Register options table with the Ace framework.
	LibStub("AceConfig-3.0"):RegisterOptionsTable(addon.name, optionsTable)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addon.name, addon.name, "Broker")


	-- Initialize our session timer, the initially displayed timer, and set the
	-- time when a new day begins (i.e., when our "today" timers get reset).
	sessionTimer = -1
	timerDisplayed = db.profile.defaultTimer
	startOfNewDay = tomorrow()
	
	-- Purge any stale timers in the saved variables database.
	if db.global.lastTimerUpdate < yesterday() then
		ResetDayTimers()
	end
	
	-- Create timers for this realm and this player, if necessary.
	CreateMyTimers()
	-- ...and then immediately decrement them, so that we don't add a minute
	-- each time we start playing.
	db.global.todayTimer = db.global.todayTimer - 1
	thisPlayer().playerTimer = thisPlayer().playerTimer - 1
	
	-- Grab "lifetime" player timers from the Blizzard "/played" slash command.
	requestingTimePlayed = true
	RequestTimePlayed();
	
	-- Start the clock on our timers and schedule an update every 60 seconds.
	self:UpdateTimers()
	self:ScheduleRepeatingTimer("UpdateTimers", 60)
end



-- addon:UpdateTimers() increments the session, today, and player timers,
-- records that the timers were up-to-date as of the current time, starts a new
-- day (if necessary), updates the tooltip if it's currently displaying, and
-- displays a reminder when it's time to display one.

function addon:UpdateTimers()
	sessionTimer = sessionTimer + 1

	if time() >= startOfNewDay then
		startOfNewDay = tomorrow()
		ResetDayTimers()
		CreateMyTimers()
	else
		db.global.todayTimer = db.global.todayTimer + 1
		thisPlayer().playerTimer = thisPlayer().playerTimer + 1
	end
	db.global.lastTimerUpdate = time()
	UpdateStatusBarText()
	
	if showingTooltip then
		GameTooltip:ClearLines()
		dataobj.OnTooltipShow(GameTooltip)
		GameTooltip:Show()
	end
	
	if sessionTimer > 0 and db.global.remindersEnabled then
		local delta = selectedTimer(db.global.reminderTimer) - db.global.remindMinutes
		if delta == 0 or (db.global.snoozeEnabled and delta > 0 and delta % db.global.snoozeMinutes == 0) then
			addon:ShowReminder()
		end
	end
	
end



-- addon:ShowReminder() displays a reminder message in the user-specified format.

function addon:ShowReminder()
	-- Sending a multi-color text string to the chat frame can cause
	-- the Blizz client to disconnect from the server, so we're calling
	-- formatTimeWithColor() with a color of nil.
	local timeString	= formatTimeWithColor(selectedTimer(db.global.reminderTimer), LONG_TIME_FORMAT, nil)
	local reminderText	= (db.global.reminderText):format(timeString)
--			local iconString	= "|T"..dataobj.icon..":0|t "

	if db.global.reminderType == RAID_WARNING then
		RaidNotice_AddMessage(RaidWarningFrame, reminderText, ChatTypeInfo["RAID_WARNING"]);
		PlaySound("RaidWarning");
		
	elseif db.global.reminderType == WHISPER then
		-- This filter temporarily stops display of outbound whispers
		-- in chat frames, since the addon-generated whisper is both
		-- sent and received by the player.
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", hideMyWhisper)
		SendChatMessage(reminderText, "WHISPER", nil, player)
		
	elseif db.global.reminderType == CHAT_FRAME then
		print(colorizeText(reminderText, CYAN))
		
	elseif db.global.reminderType == DIALOG_BOX then
		local acceptButtonString = L["Okay"]
		if db.global.snoozeEnabled then
			acceptButtonString = L["Snooze"] 
		end
		StaticPopupDialogs["PlayTimer_Reminder"] = {
			text = reminderText,
			button1 = acceptButtonString,
			button2 = L["Turn Off"],
			button3 = L["Options"],
			OnAccept = function() end,
			OnCancel = function() db.global.remindersEnabled = false end,
			OnAlt    = function() InterfaceOptionsFrame_OpenToCategory(addon.optionsFrame) end,
			timeout = 0,
			whileDead = true,
			hideOnEscape = true,
			showAlert = true,
			enterClicksFirstButton = true
		}
		StaticPopup_Show("PlayTimer_Reminder");
		
	elseif db.global.reminderType == LIBSINK then
		self:Pour(reminderText, unpack(CYAN))			
	end
end



-- dataobj:OnTooltipShow() populates the content of the tooltip.

function dataobj:OnTooltipShow()
	self:AddLine(colorizeText(L[addon.name], ORANGE))
	self:AddLine(" ")
	
	-- Player
	self:AddLine(colorizeText(player, WHITE))
	self:AddDoubleLine("  "..L["This Session"], formatTime(sessionTimer))
	self:AddDoubleLine("  "..L["Today"], formatTime(thisPlayer().playerTimer))
	self:AddDoubleLine("  "..L["Level"].." "..UnitLevel(player), formatTime(playedLevel + sessionTimer))
	self:AddDoubleLine("  "..L["Lifetime"], formatTime(playedTotal + sessionTimer))
	self:AddLine(" ")

	-- Today
	self:AddLine(colorizeText(L["Today"], WHITE))
	
	-- First pass: Calculate values for the realm and faction timers by adding
	-- up the player timers for each realm / faction.
	local numberOfRealms, factionTable, realmIndent = 0, {}, ""
	for realmName, theRealm in pairs(db.global.realmTable) do
		numberOfRealms = numberOfRealms + 1
		theRealm.realmTimer = 0
		for playerName, thePlayer in pairs(theRealm.playerTable) do
			theRealm.realmTimer = theRealm.realmTimer + thePlayer.playerTimer
			if not factionTable[thePlayer.playerFaction] then
				factionTable[thePlayer.playerFaction] = thePlayer.playerTimer
			else
				factionTable[thePlayer.playerFaction] = factionTable[thePlayer.playerFaction] + thePlayer.playerTimer
			end
		end
	end
	
	-- Second pass: Display realm, character and (optionally) faction timers.
	-- Realm timers are only displayed if there's more than one realm to show.
	if numberOfRealms > 1 then
		realmIndent = "  "
	end
	
	for realmName, theRealm in pairs(db.global.realmTable) do
		if numberOfRealms > 1 then
			self:AddDoubleLine("  "..colorizeText(realmName, LTBLUE), formatTime(theRealm.realmTimer))
		end
		for playerName, thePlayer in pairs(theRealm.playerTable) do
			self:AddDoubleLine("  "..realmIndent..colorizeText(playerName, factionColor[thePlayer.playerFaction]), formatTime(thePlayer.playerTimer))
		end
	end
	
	if db.profile.showFactions == ALWAYS or (db.profile.showFactions == IF_BOTH_PLAYED and factionTable["Alliance"] and factionTable["Horde"]) then
		self:AddLine(" ")
		self:AddDoubleLine(colorizeText("  "..L["Alliance"], factionColor["Alliance"]), formatTime(factionTable["Alliance"]))
		self:AddDoubleLine(colorizeText("  "..L["Horde"],    factionColor["Horde"   ]), formatTime(factionTable["Horde"   ]))	
	end
	
	self:AddLine(" ")
	self:AddDoubleLine("  "..L["Total Played Today"], formatTime(db.global.todayTimer))
		
end



-- dataobj:OnEnter() and OnLeave() show and hide the tooltip.

function dataobj:OnEnter()
	showingTooltip = true
	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	GameTooltip:ClearLines()
	dataobj.OnTooltipShow(GameTooltip)
	GameTooltip:Show()
end

function dataobj:OnLeave()
	showingTooltip = false
	GameTooltip:Hide()
end



-- OnClick() handles mouse clicks in the status bar:
--   Click:			Cycles session, today, and player timers.
--   Shift-click:	Cycles short, medium and long time formats.
--   Ctrl-click:	Toggles display of the timer name.
--   Right-click:	Displays the interface options dialog.

function dataobj:OnClick(button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(addon.optionsFrame)
	elseif IsShiftKeyDown() then
		db.profile.timeFormat = (db.profile.timeFormat % 3) + 1
	elseif IsControlKeyDown() then
		db.profile.showTimerName = not db.profile.showTimerName
	else
		timerDisplayed = (timerDisplayed % 3) + 1
	end
	UpdateStatusBarText()
end